import sys

sys.path.append('..')

import unittest

from square_calculator_library.square_calculator import constants
from square_calculator_library.square_calculator import square


class TestSquareCalculator(unittest.TestCase):

    def test_ability_transmit_only_positive_numbers(self):
        values_results = (
            # если встречаются отрицательные значения
            # при передаче одного параметра
            ({'a': -500}, ValueError),
            # при передаче двух параметров
            ({'a': 1, 'b': -500}, ValueError),
            # при передаче трёх параметров
            ({'a': 1, 'b': -500, 'c': 50}, ValueError),
            # при передаче четырёх параметров
            ({'a': 1, 'b': -500, 'c': 50, 'd': -10}, ValueError),
            # если встречаются нулевые значения
            # при передаче одного параметра
            ({'a': 0}, ValueError),
            # при передаче двух параметров
            ({'a': 0, 'b': -500}, ValueError),
            # при передаче трёх параметров
            ({'a': 0, 'b': -500, 'c': 50}, ValueError),
            # при передаче четырёх параметров
            ({'a': 0, 'b': -500, 'c': 50, 'd': -10}, ValueError),
        )
        for value, expected_error in values_results:
            with self.subTest():
                with self.assertRaises(expected_error):
                    sq = square.Square(**value)
                    sq.calculate()

    def test_square_calculating(self):
        def verification_triangle_square(*args):
            half_perimeter = sum(args) / 2
            return (
                half_perimeter
                * (half_perimeter - args[0])
                * (half_perimeter - args[1])
                * (half_perimeter - args[2])
            ) ** 0.5

        def varification_circle_square(radius):
            return constants.PI * radius ** 2

        def varification_ellipse_square(a, b):
            return constants.PI * a * b

        def varification_rectangle_square(a, b):
            return a * b

        def varification_trapezoid_square(a, b, h):
            return 0.5 * (a + b) * h

        values_results = (
            # при передаче одного параметра - окружность
            ({'a': 10}, varification_circle_square(10)),
            # при передаче двух параметров для элипса
            (
                {'a': 10, 'b': 500, 'figure_type': constants.FIGURE_TYPES[1]},
                varification_ellipse_square(10, 500)
            ),
            # при передаче двух параметров для прямоугольника
            (
                {'a': 10, 'b': 500, 'figure_type': constants.FIGURE_TYPES[2]},
                varification_rectangle_square(10, 500)
            ),
            # при передаче двух параметров и высоты для трапеции
            (
                {
                    'a': 10, 'b': 500, 'h': 50,
                    'figure_type': constants.FIGURE_TYPES[4]
                },
                varification_trapezoid_square(10, 500, 50)
            ),
            # при передаче трёх параметров для треугольника
            (
                {'a': 10, 'b': 500, 'с': 50},
                verification_triangle_square(10, 500, 50)
            ),
        )
        for value, expected_result in values_results:
            with self.subTest(
                value=value,
                expected_result=expected_result,
                msg=(
                    f'При передаче значений {value},\n'
                    f'результат отличался от ожидаемого: {expected_result}.'
                )
            ):
                values = {
                    index: v for index, v in value.items()
                    if index not in [
                        'figure_type',
                        'h'
                    ]
                }
                sq = square.Square(**values)
                if any(attr == 'figure_type' for attr in value.keys()):
                    sq.figure_type = value['figure_type']
                if any(attr == 'h' for attr in value.keys()):
                    sq.trapezoid_h = value['h']
                result = sq.calculate()

                self.assertEqual(result, expected_result)

    def test_figure_type_definitions(self):
        values_results = (
            # при передаче одного параметра - окружность
            ({'a': 10}, constants.FIGURE_TYPES[0]),
            # при передаче двух параметров для элипса
            (
                {'a': 10, 'b': 500, 'figure_type': constants.FIGURE_TYPES[1]},
                constants.FIGURE_TYPES[1]
            ),
            # при передаче двух параметров для прямоугольника
            (
                {'a': 10, 'b': 500, 'figure_type': constants.FIGURE_TYPES[2]},
                constants.FIGURE_TYPES[2]
            ),
            # при передаче двух параметров и высоты для трапеции
            (
                {
                    'a': 10, 'b': 500, 'h': 50,
                    'figure_type': constants.FIGURE_TYPES[4]
                },
                constants.FIGURE_TYPES[4]
            ),
            # при передаче трёх параметров для треугольника
            (
                {'a': 10, 'b': 500, 'с': 50},
                constants.FIGURE_TYPES[5]
            ),
            # при передаче трёх параметров для прямоугольного треугольника
            (
                {'a': 3, 'b': 4, 'с': 5},
                constants.FIGURE_TYPES[3]
            ),
        )
        for value, expected_result in values_results:
            with self.subTest(
                value=value,
                expected_result=expected_result,
                msg=(
                    f'При передаче значений {value},\n'
                    f'результат отличался от ожидаемого: {expected_result}.'
                )
            ):
                values = {
                    index: v for index, v in value.items()
                    if index not in [
                        'figure_type',
                        'h'
                    ]
                }
                sq = square.Square(**values)
                if any(attr == 'figure_type' for attr in value.keys()):
                    sq.figure_type = value['figure_type']
                if any(attr == 'h' for attr in value.keys()):
                    sq.trapezoid_h = value['h']
                sq.calculate()
                result = sq.figure_type

                self.assertEqual(result, expected_result)

    def test_assert_for_not_circle_or_triangle_without_figure_type(self):
        values_results = (
            # без передачи параметров
            ({}, Exception),
            # при передаче двух параметров
            ({'a': 1, 'b': 500}, Exception),
            # при передаче четырёх параметров
            ({'a': 1, 'b': 500, 'c': 50, 'd': 10}, Exception),
        )
        for value, expected_error in values_results:
            with self.subTest():
                with self.assertRaises(expected_error):
                    sq = square.Square(**value)
                    sq.calculate()


unittest.main()



from . import constants as c
from . import exception_messages as m


class Square:
    """
    Рассчитывает площадь геометрической фигуры.
    По умолчанию рассчитывает площадь
    треугольника по трем сторонам
    или окружности по радиусу.

    Для корректной работы библиотеки
    при создании объекта класса
    необходимо передавать именованые аргументы,
    имена могут быть произвольными.

    Так нельзя:

    sq = Square(3, 4, 5)

    Так можно:

    sq = Square(a=3, b=4, c=5)

    Если передать в аргументах тип фигуры:
    'ellipse', 'rectangle' или 'trapezoid'
    рассчитывает площадь соответственно
    элипса, прямоугольника или трапеции.

    Тип фигур можно задавать,
    используя кортеж FIGURE_TYPES из файла constants.

    Определить тип рассчитаной фигуры можно через атрибут figure_type,
    например, так:

    sq = Square(a=3, b=4, c=5)

    print(sq.figure_type)

    >> 'rectangular triangle'
    """

    figure_type = None
    trapezoid_h = None

    def __init__(self, **kwargs):
        if self.validate(**kwargs):
            self.__dict__.update(kwargs)

    def validate(self, **kwargs):
        if any(i <= 0 for i in kwargs.values() if type(i) is int):
            raise ValueError('Длины всех сторон должны быть больше 0.')
        else:
            return True

    def get_attributes(self):
        return [
            value for index, value in self.__dict__.items() if index not in [
                'figure_type',
                'trapezoid_h'
            ]
        ]

    def count_attributes(self) -> int:
        return len(self.get_attributes())

    def is_rectangular(self) -> bool:
        """Вернет True, если стороны образуют прямоугольный треугольник."""
        sides = self.get_attributes()
        results = []
        for i, side in enumerate(sides):
            other_indexes = [
                index for index, value in enumerate(sides) if index != i
            ]
            if side ** 2 == (
                sides[other_indexes[0]] ** 2 + sides[other_indexes[1]] ** 2
            ):
                results.append(True)
            else:
                results.append(False)
        return any(results)

    def triangle(self):
        params = self.get_attributes()
        self.figure_type = c.FIGURE_TYPES[5]
        half_perimeter = sum(params) / 2
        return (
            half_perimeter
            * (half_perimeter - params[0])
            * (half_perimeter - params[1])
            * (half_perimeter - params[2])
        ) ** 0.5

    def rectangular_triangle(self):
        params = self.get_attributes()
        self.figure_type = 'rectangular triangle'
        exclude = max(params)
        katets = [side for side in params if side != exclude]
        return 0.5 * katets[0] * katets[1]

    def circle(self):
        self.figure_type = 'circle'
        return c.PI * self.get_attributes()[0] ** 2

    def ellipse(self):
        params = self.get_attributes()
        self.figure_type = 'ellipse'
        return c.PI * params[0] * params[1]

    def rectangle(self):
        params = self.get_attributes()
        self.figure_type = 'rectangle'
        return params[0] * params[1]

    def trapezoid(self):
        self.figure_type = 'trapezoid'
        return 0.5 * sum(self.get_attributes()) * self.trapezoid_h

    def calculate(self) -> float:
        attrs_count = self.count_attributes()
        if self.figure_type is None:
            if attrs_count not in [1, 3]:
                raise Exception(
                    m.ONE_OR_THREE_ARGS_REQUIRED
                )
            if attrs_count == 1:
                result = self.circle()
            if attrs_count == 3:
                if self.is_rectangular():
                    result = self.rectangular_triangle()
                else:
                    result = self.triangle()
        elif self.figure_type == c.FIGURE_TYPES[1]:
            if attrs_count == 2:
                result = self.ellipse()
            else:
                raise Exception(m.TWO_ARGS_REQUIRED)
        elif self.figure_type == c.FIGURE_TYPES[2]:
            if attrs_count == 2:
                result = self.rectangle()
            else:
                raise Exception(m.TWO_ARGS_REQUIRED)
        elif self.figure_type == c.FIGURE_TYPES[4]:
            if attrs_count == 2 and self.trapezoid_h is not None:
                result = self.trapezoid()
            else:
                raise Exception(m.TWO_ARGS_AND_TRAPEZOID_H_REQUIRED)
        else:
            raise Exception(m.UNKNOWN_ERROR)
        return result



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

requirements = [
    "flake8==6.0.0",
    "flake8-isort==6.0.0",
    "isort==5.13.2",
    "mccabe==0.7.0",
    "pycodestyle==2.10.0",
    "pyflakes==3.0.1",
]

setuptools.setup(
    # Имя дистрибутива пакета. Оно должно быть уникальным, поэтому добавление вашего имени пользователя в конце является обычным делом.
    name="square_calculator_library",
    # Номер версии вашего пакета. Обычно используется семантическое управление версиями.
    version="0.0.1",
    # Имя автора.
    author="Aleksandr Buchelnikov",
    # Его почта.
    author_email="al.buchelnikov@gmail.com",
    # Краткое описание, которое будет показано на странице PyPi.
    description="The library allows you to calculate the area of geometric shapes.",
    # Длинное описание, которое будет отображаться на странице PyPi. Использует README.md репозитория для заполнения.
    long_description=long_description,
    # Определяет тип контента, используемый в long_description.
    long_description_content_type="text/markdown",
    # URL-адрес, представляющий домашнюю страницу проекта. Большинство проектов ссылаются на репозиторий.
    url="https://github.com/AVanslov/square_calculator_library",
    # Находит все пакеты внутри проекта и объединяет их в дистрибутив.
    packages=setuptools.find_packages(),
    # requirements или dependencies, которые будут установлены вместе с пакетом, когда пользователь установит его через pip.
    install_requires=requirements,
    # Предоставляет pip некоторые метаданные о пакете. Также отображается на странице PyPi.
    classifiers=[
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    # Требуемая версия Python.
    python_requires='>=3.10',
)



ONE_OR_THREE_ARGS_REQUIRED = (
    'Передано некоректное количество аргументов. '
    'Их должно быть 1 или 3.'
)
TWO_ARGS_REQUIRED = (
    'Передано некоректное количество аргументов. '
    'Их должно быть 2.'
)
TWO_ARGS_AND_TRAPEZOID_H_REQUIRED = (
    'Передано некоректное количество аргументов.'
    'Должны быть переданы 2 размера:'
    'длинна оснований трапеции,'
    'а также её высота - параметр trapezoid_h.'
)
UNKNOWN_ERROR = (
    'Неизвестная ошибка выполнения программы.'
)
